@@ -8,14 +8,17 @@
 #'             group (numeric, character or factor); __or__, a data frame with binary data
 #'             (must have columns `a`, `c`, `b`/`n1`, `d`/`n2`).
 #' @param effect what effect to calculate? a `mean` (and SE) of outcome in groups or
-#'               (for binary data) `logOR` (odds ratio), `logRR` (risk ratio);
+#'               (for binary data) `logOR` (odds ratio), `logRR` (risk ratio),
+#'               `RD` (risk difference);
 #' @param log logical; log-transform the outcome variable?
-#' @param rare_event_correction If effect is `logOR` or `logRR`, this correction
-#'             is used when working with
-#'             binary data only. The value of correction is added to all arms
-#'             in trials where some arms had 0 events.
+#' @param rare_event_correction This correction is used when working with
+#'             binary data (effect `logOR` or `logRR`)
+#'             The value of correction is added to all cells in
+#'             either some or all rows (groups), depending on `correction_type`.
 #'             Using corrections may bias results but is the only alternative to
 #'             avoid infinite values.
+#' @param correction_type If `"single"` then rare event correction is only applied to the
+#'             particular rows that have 0 cells, if `"all"`, then to all studies
 #' @param cfb logical; calculate change from baseline? If yes, the outcome
 #'            variable is taken as a difference between values in `outcome` and
 #'            `baseline` columns
@@ -25,6 +28,7 @@
 #' @param outcome name of column with outcome variable
 #' @param treatment name of column with treatment variable
 #' @param baseline name of column with baseline variable
+#' @param pooling Internal use only, please ignore
 #'
 #' @return
 #' * If you `summarise`: a data.frame with columns for `group`, `tau` and `se.tau`
@@ -48,28 +52,48 @@
 #'
 #' @author Witold Wiecek
 #' @seealso [convert_inputs] for how any type of data is (internally) converted into
-#'          a list of Stan inputs;
+#'          a list of Stan inputs; vignette `baggr_binary` for more details about
+#'          rare event corrections
 #' @export
 #' @import stats
 #'
+#' @examples
+#'
+#' # Example of working with binary outcomes data
+#' # Make up some individual-level data first:
+#' df_rare <- data.frame(group = paste("Study", LETTERS[1:5]),
+#'                       a = c(0, 2, 1, 3, 1), c = c(2, 2, 3, 3, 5),
+#'                       n1i = c(120, 300, 110, 250, 95),
+#'                       n2i = c(120, 300, 110, 250, 95))
+#' df_rare_ind <- binary_to_individual(df_rare)
+#' # Calculate ORs; default rare event correction will be applied
+#' prepare_ma(df_rare_ind, effect = "logOR")
+#' # Add 0.5 to all rows
+#' prepare_ma(df_rare_ind, effect = "logOR",
+#'            correction_type = "all",
+#'            rare_event_correction = 0.5)
 
 prepare_ma <- function(data, #standardise = NULL,
-                       effect = c("mean", "logOR", "logRR"),
+                       effect = c("mean", "logOR", "logRR", "RD"),
                        rare_event_correction = 0.25,
+                       correction_type = c("single", "all"),
                        log = FALSE, cfb = FALSE, summarise = TRUE,
                        treatment="treatment",
                        baseline = NULL,
                        group="group",
-                       outcome="outcome") {
+                       outcome="outcome",
+                       pooling=FALSE) {
+
+  effect <- match.arg(effect)
+  correction_type <- match.arg(correction_type)
 
-  effect <- match.arg(effect, c("mean", "logOR", "logRR"))
   if(grepl("pool|unknown", detect_input_type(data, group, treatment, outcome))){
-    if(effect %in% c("logOR", "logRR")){
+    if(effect %in% c("logOR", "logRR", "RD")){
       check_columns_binary(data)
       data <- binary_to_individual(data, group)
       group <- "group"
     }else
-      stop("Data must be individual-level (if summarising) or binary data (if converting), see ?prepare_ma")
+      stop("Data must be individual-level (if summarising) or binary (if converting), see ?prepare_ma")
   }
 
   check_columns(data, outcome, group, treatment, stop.for.na = FALSE)
@@ -89,7 +113,7 @@ prepare_ma <- function(data, #standardise = NULL,
       check_columns(data, outcome, group, treatment, stop.for.na = TRUE)
   }
 
-  if(effect %in% c("logOR", "logRR")) {
+  if(effect %in% c("logOR", "logRR", "RD")) {
     if(!is_binary(data$outcome))
       stop("Outcome column is not binary (only 0 and 1 values allowed).")
   }
@@ -172,17 +196,23 @@ prepare_ma <- function(data, #standardise = NULL,
                                idvar = "group", direction = "wide")
       sewide <- stats::reshape(data = seagg, timevar = "treatment",
                                idvar = "group", direction = "wide")
+      nagg  <- stats::aggregate(outcome ~ treatment + group, length, data = data)
+      nwide  <- stats::reshape(data = nagg, timevar = "treatment",
+                               idvar = "group", direction = "wide")
+
       out <- data.frame(group = mwide$group,
                         mu = mwide$outcome.0,
                         tau = mwide$outcome.1 - mwide$outcome.0,
                         se.mu = sewide$outcome.0,
-                        se.tau = sqrt(sewide$outcome.0^2 + sewide$outcome.1^2))
+                        se.tau = sqrt(sewide$outcome.0^2 + sewide$outcome.1^2),
+                        n.mu = nwide$outcome.0,
+                        n.tau = nwide$outcome.1)
     }
 
     # Prepare event counts for binary data models
     # (including rare event corrections)
-    if(effect %in% c("logOR", "logRR")) {
-      v <- rare_event_correction
+    if(effect %in% c("logOR", "logRR", "RD")) {
+
       binary_data_table <-
         do.call(rbind, by(data, list(data$group), function(x) {
           with(x,
@@ -195,29 +225,14 @@ prepare_ma <- function(data, #standardise = NULL,
                  b     = sum(treatment == 1) - sum(outcome[treatment == 1]),
                  d     = sum(treatment == 0) - sum(outcome[treatment == 0])))
         }))
-      rare <- with(binary_data_table, (a == 0 | b == 0 | c == 0 | d == 0))
-      if(sum(rare) > 0 && rare_event_correction == 0.25)
-        message("Applied default rare event correction (0.25) in ", sum(rare), " studies")
 
-      binary_data_table$a  <- v*rare + binary_data_table$a
-      binary_data_table$b  <- v*rare + binary_data_table$b
-      binary_data_table$c  <- v*rare + binary_data_table$c
-      binary_data_table$d  <- v*rare + binary_data_table$d
-      binary_data_table$n1 <- v*rare + binary_data_table$n1
-      binary_data_table$n2 <- v*rare + binary_data_table$n2
-
-
-      out <- binary_data_table
+      out <- apply_cont_corr(binary_data_table,
+                             rare_event_correction,
+                             correction_type,
+                             pooling=pooling)
       rownames(out) <- NULL
-
-      if(effect == "logRR") {
-        out$tau <- with(out, log((a/(a+b))/(c/(c+d))))
-        out$se  <- with(out, sqrt(1/a + 1/c - 1/(a+b) - 1/(c+d)))
-      }
-      if(effect == "logOR") {
-        out$tau <- with(out, log((a*d)/(b*c)))
-        out$se  <- with(out, sqrt(1/a + 1/b + 1/c + 1/d))
-      }
+      # Add OR or RR estimate
+      out <- calc_or_rr(out, effect)
 
     }
   } else {
@@ -227,3 +242,59 @@ prepare_ma <- function(data, #standardise = NULL,
   out
 }
 
+# see prepare_ma()
+calc_or_rr <- function(out, effect) {
+  if(effect == "logRR") {
+    out$tau <- with(out, log((a/(a+b))/(c/(c+d))))
+    out$se  <- with(out, sqrt(1/a + 1/c - 1/(a+b) - 1/(c+d)))
+  }
+  if(effect == "logOR") {
+    out$tau <- with(out, log((a*d)/(b*c)))
+    out$se  <- with(out, sqrt(1/a + 1/b + 1/c + 1/d))
+  }
+  if(effect == "RD") {
+    out$tau <- with(out, (a/(a+b)) - (c/(c+d)))
+    out$se  <- with(out, {
+      p1 <- a/(a+b); p2 <- c/(c+d);
+      v1 <- p1*(1-p1)/(a+b)
+      v2 <- p2*(1-p2)/(c+d)
+      sqrt(v1 + v2)
+    })
+
+  }
+  out
+}
+
+# see prepare_ma()
+apply_cont_corr <- function(binary_data_table, v, correction_type,
+                            add_or = FALSE,
+                            pooling = FALSE #indicates that this was called to calc pooling
+) {
+  rare_event_correction <- v
+  rare <- with(binary_data_table, (a == 0 | b == 0 | c == 0 | d == 0))
+
+  if(sum(rare) == 1 && rare_event_correction == 0.25 && pooling == 0)
+    message("Applied default rare event correction (0.25) in 1 study")
+  if(sum(rare) > 1 && rare_event_correction == 0.25  && pooling == 0)
+    message("Applied default rare event corrections (0.25) in ", sum(rare), " studies")
+  # if(sum(rare) > 0 && pooling == 1)
+  #   message("Some pooling values calculated using rare event corrections.")
+
+  if(correction_type == "single")
+    cc_value <- v*rare
+  if(correction_type == "all")
+    cc_value <- rare_event_correction
+
+  binary_data_table$a  <- cc_value + binary_data_table$a
+  binary_data_table$b  <- cc_value + binary_data_table$b
+  binary_data_table$c  <- cc_value + binary_data_table$c
+  binary_data_table$d  <- cc_value + binary_data_table$d
+  binary_data_table$n1 <- cc_value + binary_data_table$n1
+  binary_data_table$n2 <- cc_value + binary_data_table$n2
+
+  if(!add_or)
+    return(binary_data_table)
+  else
+    return(calc_or_rr(binary_data_table, "logOR"))
+}
+
